% Scrip to generate a broadband transtion from on-PCB microstrip to
% integrated air-filled waveguide having a smooth impedance taper
%
% Suplementary material to the paper:
% J. Sorocki, I. Piekarz, N. Delmonte, L. Silvestri, and M. Bozzi,
% "Leveraging Additive Manufacturing for Realization of Low-Loss Integrated Printed Circuit Board-Waveguide Stack-up for Highly Integrated Circuits Application"
%
% Written by:
% Jakub Sorocki, PhD, jakub.sorocki@agh.edu.pl
% Ilona Piekarz, PhD, ilona.piekarz@agh.edu.pl
% Microwave Research Group, www.microwaves.agh.edu.pl
% Institute of Electrinics, AGH University of Science and Technology,
% Poland
%
% Revision: rev1 winter 2021

close all
clear all
clc

eps0=8.85418782*1e-12;
mu0=1.25663706*1e-6;
c0=1/sqrt(eps0*mu0);
eta0=sqrt(mu0/eps0);

%% -- Plotting and saving settings --
WRITE_DXF = 0;
dim_prec = 2; % dimension precision, num of significant digits, 3 - down to 1um
f_name_postfix = '_0.27_0.8';

ENABLE_SAVE_PLOT = 0;
plo.fPath_save = '';
plo.Sx=20; plo.Sy=12; % fig size
plo.Xlimits = [0 18 2];   % freq limits in (GHz)
plo.Ylimits_perm = [0 18 3];   % freq limits in (GHz)
plo.fontSize = 32;
plo.lineWidth = 2.5;
plo.lineColor = lines(4);
plo.markerVect = ["s","v","o","^","<",">","*","x","|""+","d","p","h","_"];
plo.markerSize = 14;
plo.markerIdx = 1:200:1800;
plo.markerSep = 50;

%% -- Design specification --
% substrate properties
er_sub = 3.5;
hsub = 0.51;    % in mm

% output waveguide properties
er_wg = 1;
hwg = 10;   % waveguide height in mm

f0 = 10.5;    % center frequency for TE10 mode in GHz
fc = f0/1.575;  % required cut-off frequency for the TE10 mode in GHz

L = 30;   % total tapper length in mm
dL = 0.15;   % length step od 0.5 mm
Ltap = 0:dL:L;

% treshold for whet the guide structure changes from suspended microstrip
% to SIW alike waveguide to air-filled waveguide
mssiw_th = 0.27;	% cutoff for ms to siw, from 0 to 1 of taper length
siwwg = 0.8;	% cutoff for top laminate, from 0 to 1

%% --- Synthesize transition geometry ---
% find MS for start
Zms = 50;

% find AF-WG for end
err2 = @(awg)err_wg_fc(awg, hwg, er_wg, fc);   % find wg geometry to meet the required cut-off frequency
x0 = 20;
aafwg = fminsearch(err2, x0);
aafwg = round(aafwg, dim_prec);
bafwg = hwg;
% Zafwg = 2.*(eta0./sqrt(er_wg)).*(bafwg./aafwg);
afwg = WG_prop(aafwg,bafwg,f0,er_wg,0);
Zafwg = afwg.Z0;
Zafwg = round(Zafwg, 1);

% find inter-via distance for the SIW part. assumption is that widht of the
% waveguide is fixed
viad = 0.5;
pvia = 1;
err = @(avia)err_siw(avia, aafwg, viad, pvia);
x0 = aafwg;
avia_wg = fminsearch(err, x0);
avia_wg = round(avia_wg, dim_prec);

%% Taper impedance profile
Z0 = Zms;      % characteristic impedance at input
ZL = Zafwg; % characteistic wave impedance at output

R_dB = -25;
Z0tap = bklop(Z0,ZL,18, R_dB);  % too high number of steps yelds strange behavour of Zklopt function and thus interpolation is used
Z0tapL = interp1([1:20],Z0tap,linspace(1,20,length(Ltap)));

disp(strcat('Klop transf Z0',32,num2str(Z0),32,', ZL',32,num2str(ZL)));
% Taper profile is returned as a list of impedances, the optimum length
% for this design of taper is 0.565 lambda.

%% Plot indexing start
plo.idx = 1;

%% Impedance taper plot
hFig(plo.idx)=figure(plo.idx);
set(hFig(plo.idx), 'Units', 'inches');
set(hFig(plo.idx), 'PaperUnits', 'inches');
hFig(plo.idx).Position = [0 0 plo.Sx plo.Sy];

% subplot(3,1,[1 2])
plot(Ltap./L, Z0tapL,'LineWidth',plo.lineWidth)
grid on
xlabel('Ltap');
ylabel('Ztap (dL)')
ylim([0 300]);
set(gca,'GridAlpha', 1);
set(gca,'FontSize', plo.fontSize);

if ENABLE_SAVE_PLOT == true
    set(hFig(plo.idx), 'Units', 'inches');
    set(hFig(plo.idx), 'PaperUnits', 'inches');
    hFig(plo.idx).Position = [0 0 plo.Sx plo.Sy];
    
    filename = strcat(plo.fPath_save,'Fig_',num2str(plo.idx),f_name_postfix);
    saveas(hFig(plo.idx), strcat(filename, '.png'), 'png');    % save permittivity plot as .png
    RemoveWhiteSpace([], 'file', strcat(filename,'.png'));
end

plo.idx = plo.idx+1;

%%
% Find idx for where to transition from one guide type to other
[~, mssiw_thidx ] = min(abs(Ltap-mssiw_th*L));
[~, siwwg_thidx ] = min(abs(Ltap-siwwg*L));

ones(1,length(Ltap));
Aafwg_tap = linspace(aafwg, aafwg, length(Ltap));
Aafwg_tap = round(Aafwg_tap, dim_prec);

% Find geometry (bwg) of SIW+WG to satisfy the impedance taper
for idx = 1:length(Ltap)
    err = @(bwg)err_swg2(Aafwg_tap(idx), bwg, hsub, er_sub, er_wg, f0, Z0tapL(idx));
    bwg = fminsearch(err, 1);
    Bsiwwgtap(idx) = bwg;
end

% Find geometry (bwg) of WG to satisfy the impedance taper
for idx = 1:length(Ltap)
    err = @(bwg)err_swg2(aafwg, bwg, 0, er_sub, er_wg, f0, Z0tapL(idx));
    bwg = fminsearch(err, 1);
    Bwgtap(idx) = bwg;
end

% Find ms width at susp ms to siw-alike transition
err = @(w)err_ms(w,hsub,er_sub, Z0);
msw = fminsearch(err, 1.2);
msw = round(msw, 2);
err = @(w)err_ms_susp(w,hsub,er_sub, Bsiwwgtap(mssiw_thidx), Z0tapL(mssiw_thidx));
if mssiw_thidx == siwwg_thidx
    err = @(w)err_ms_susp(w,hsub,er_sub, Bwgtap(siwwg_thidx), Z0tapL(siwwg_thidx));
end
mssiw_thw = fminsearch(err, 1);

Wmstap2 = [linspace(msw, mssiw_thw, mssiw_thidx) aafwg.*ones(1, length(Ltap)-mssiw_thidx)];
Htap1 = (Bsiwwgtap(mssiw_thidx)./Ltap(mssiw_thidx)).*Ltap;
if mssiw_thidx == siwwg_thidx
    Htap1 = (Bwgtap(siwwg_thidx)./Ltap(siwwg_thidx)).*Ltap;
end

% Find geometry (w) of susp MS to satisfy the impedance taper
for idx = 1:length(Ltap)
    err = @(w)err_ms_susp(w,hsub,er_sub, Htap1(idx), Z0tapL(idx));
    w = fminsearch(err, 1);
    Wmstap1(idx) = w;
end

%
Btran_er_eff = eff_perm(er_sub, hsub, er_wg, Bsiwwgtap);
Btran_fc_10 = (c0./(2.*pi.*sqrt(Btran_er_eff))).*(pi./Aafwg_tap)./1e6;

%% Synthesize taper for top metal width (Wmstap) and waveguide thickness
% (Btran) using the above calculated geometry of intermediate guides
Wms_tap = [msw Wmstap1(2:mssiw_thidx) Aafwg_tap(mssiw_thidx+1:siwwg_thidx) aafwg.*ones(1, length(Ltap)-siwwg_thidx)];
Btran_tap = [Htap1(1:mssiw_thidx) Bsiwwgtap(mssiw_thidx+1:siwwg_thidx) Bwgtap(siwwg_thidx+1:end-1) hwg];


%% Taper geometry plot
hFig(plo.idx)=figure(plo.idx);
set(hFig(plo.idx), 'Units', 'inches');
set(hFig(plo.idx), 'PaperUnits', 'inches');
hFig(plo.idx).Position = [0 0 plo.Sx plo.Sy];

subplot(3,1,1)
grid on
hold on

fccc = [fc.*Htap1(1:mssiw_thidx)./Htap1(1:mssiw_thidx) Btran_fc_10(mssiw_thidx+1:siwwg_thidx) fc.*Bwgtap(siwwg_thidx+1:end)./Bwgtap(siwwg_thidx+1:end)];
plot(Ltap(mssiw_thidx:end), fccc(mssiw_thidx:end),'LineWidth',plo.lineWidth);
xline(mssiw_th*L,'LineWidth',plo.lineWidth);
xline(siwwg*L,'LineWidth',plo.lineWidth);
set(gca,'GridAlpha', 1);
set(gca,'FontSize', plo.fontSize);
ylim([0.9*fc,1.1*fc]);
xlabel('Ltap (mm)');
ylabel('fc_{TE10} (GHz)');


subplot(3,1,[2 3])
hold on
grid on

plot(Ltap, Wms_tap,'LineWidth',plo.lineWidth);
plot(Ltap, Btran_tap,'LineWidth',plo.lineWidth)

plot(Ltap, Wmstap1,'--');
plot(Ltap, Htap1,'--');
plot(Ltap, Bsiwwgtap,'--');
plot(Ltap, Bwgtap,'--');

xline(mssiw_th*L,'LineWidth',plo.lineWidth)
xline(siwwg*L,'LineWidth',plo.lineWidth)

xlabel('Ltap (mm)');
ylabel('dim (mm)');
legend('Wtap','Btap');
ylim([0 ceil(aafwg)+1]);
% ylim([0 25])

set(gca,'GridAlpha', 1);
set(gca,'FontSize', plo.fontSize);

if ENABLE_SAVE_PLOT == true
    set(hFig(plo.idx), 'Units', 'inches');
    set(hFig(plo.idx), 'PaperUnits', 'inches');
    hFig(plo.idx).Position = [0 0 plo.Sx plo.Sy];
    
    filename = strcat(plo.fPath_save,'Fig_',num2str(plo.idx),f_name_postfix);
    saveas(hFig(plo.idx), strcat(filename, '.png'), 'png');    % save permittivity plot as .png
    RemoveWhiteSpace([], 'file', strcat(filename,'.png'));
end

plo.idx = plo.idx + 1;

%% Find phase shift for taper lower cut-off
% final B tapper
for idx = 1:length(Ltap)
    if idx == 1
        [Z0_tap(idx), epseff_tap(idx)] = ms_Hammerstad_Kirschning(msw, hsub, er_sub);
    elseif idx <=  mssiw_thidx && idx > 1
        [Z0_tap(idx), epseff_tap(idx)] = ms_susp2_impedance(Wmstap1(idx), hsub, er_sub, Htap1(idx));
    elseif idx <= siwwg_thidx && idx > mssiw_thidx
        epseff_tap(idx) = eff_perm(er_sub, hsub, er_wg, Bsiwwgtap(idx));
        Z0_tap(idx) = 2.*(eta0./sqrt(epseff_tap(idx))).*((Btran_tap(idx)+hsub)./Wms_tap(idx));
    else
        epseff_tap(idx) = eff_perm(er_sub, 0, er_wg, Bwgtap(idx));
        Z0_tap(idx) = 2.*(eta0./sqrt(1)).*((Btran_tap(idx))./Wms_tap(idx));
    end
end

Klo_opt_ph = 0.565.*2.*pi; % of lambda guided
f_co = (Klo_opt_ph.*c0)./(2.*pi.*sum(sqrt(epseff_tap).*dL.*1e-3));
disp(strcat('Klop transf fc',32,num2str(f_co/1e9),32,'GHz'));

tan_th = sum(sqrt(epseff_tap).*dL.*1e-3).*(2.*pi./c0);
disp(strcat('phase @',32,num2str(f0),32,'GHz',32,num2str(rad2deg(tan_th*f0*1e9)),32,'deg;', 32, 'opt',32,num2str(rad2deg(Klo_opt_ph))));


%% --- Export taper geometry to dxf ---
if WRITE_DXF == true
    filename = strcat('tran_','MS2SIW2WG','_',num2str(Z0),'_',num2str(round(ZL,0)),'_',num2str(L),'_sub_',num2str(hsub),'_',num2str(er_sub),'_Nste_',num2str(length(Ltap)),f_name_postfix,'.dxf');
    
    % wgprofile
    XpolyT=[0,Ltap,fliplr(Ltap),0];
    YpolyT=[0,Btran_tap,zeros(1,length(Ltap)),0];
    ZpolyT=zeros(size(XpolyT));
    
    % msline
    XpolyC=[0,Ltap,fliplr(Ltap),0];
    YpolyC=[0,Wms_tap./2,-fliplr(Wms_tap./2),0];
    ZpolyC=zeros(size(XpolyC));
    
    layername='TRANSITION';
    pcolour=1; % 1=red,2=yellow,3=green,4=cyan,5=blue,6=magenta
    units=4; % 0 = Unitless; 1 = Inches; 4 = Millimeters; 5 = Centimeters; 6 = Meters; 8 = Microinches;
    mat2dxfp([XpolyT,-1e6,XpolyC],[YpolyT,-1e6,YpolyC],[ZpolyT,-1e6,ZpolyC],pcolour,layername,filename, units);
end


%% --- MISC FUNCTIONS ---

function e = err_wg_fc(awg, hsub, er_eff, fc)
eps0=8.85418782*1e-12;
mu0=1.25663706*1e-6;
c0=1/sqrt(eps0*mu0);
eta0=sqrt(mu0/eps0);

fc_10 = (c0./(2.*pi.*sqrt(er_eff))).*(pi./awg)./1e6;
e = abs(fc_10-fc);
end

function e = err_swg(vect, hsub, er_sub, er_wg, fc, Z0)
awg = vect(1);
bwg = vect(2);

er_eff = 1./( ((1./er_sub).*(hsub./(hsub+bwg))) + ((1./er_wg).*(bwg./(hsub+bwg))) );

eps0=8.85418782*1e-12;
mu0=1.25663706*1e-6;
c0=1/sqrt(eps0*mu0);
eta0=sqrt(mu0/eps0);

Zwg = 2.*(eta0./sqrt(er_eff)).*((bwg+hsub)./awg);
fc_10 = (c0./(2.*pi.*sqrt(er_eff))).*(pi./awg)./1e6;


if fc_10 < 4 || fc_10 > 8 || Zwg <= 5 || Zwg > 500 || awg < 1 || awg > 30 || bwg < 0
    er = 500;
else
    er = 0;
end
e = abs((Zwg-Z0)+10*(fc_10-fc)+er);
end

function e = err_swg_Z_fc(awg, bwg, hsub, er_sub, er_wg, fc, Z0)

er_eff = eff_perm(er_sub, hsub, er_wg, bwg);

eps0=8.85418782*1e-12;
mu0=1.25663706*1e-6;
c0=1/sqrt(eps0*mu0);
eta0=sqrt(mu0/eps0);

Zwg = 2.*(eta0./sqrt(er_eff)).*((bwg+hsub)./awg);
fc_10 = (c0./(2.*pi.*sqrt(er_eff))).*(pi./awg)./1e6;

if fc_10 < 5 || fc_10 > 11 || Zwg < 5 || Zwg > 1000 || awg < 1 || bwg < 0
    er = 1000;
else
    er = 0;
end
e = abs((Zwg-Z0)+2*(fc_10-fc)+er);
end

function e = err_swg2(awg, bwg, hsub, er_sub, er_wg, f0, Z0)
er_eff = eff_perm(er_sub, hsub, er_wg, bwg);
eps0=8.85418782*1e-12;
mu0=1.25663706*1e-6;
c0=1/sqrt(eps0*mu0);
eta0=sqrt(mu0/eps0);
% Zwg = 2.*(eta0./sqrt(er_eff)).*((bwg+hsub)./awg);

wg = WG_prop(awg,bwg+hsub,f0,er_eff,0);
Zwg = wg.Z0;


if Zwg < 5 || Zwg > 1000 || awg < 1
    er = 1000;
else
    er = 0;
end
e = abs(Zwg-Z0);
end

function e = err_siw(avia, asiw, viad, pvia)
aeff = avia - 1.08*((viad^2)/pvia) + 0.1*((viad^2)/avia);
e = abs(aeff - asiw);
end

function e = err_ms_susp(w,a, epsr, b, Z)
e = abs(ms_susp2_impedance(w,a, epsr, b) - Z);
end

function e = err_ms(w,hsub, epsr, Z)
w = w.*1e-3;
hsub = hsub.*1e-3;
e = abs(ms_Hammerstad_Kirschning(w, hsub, epsr) - Z);
end

function er_eff = eff_perm(er_sub, hsub, er_wg, bwg)
er_eff = 1./( ((1./er_sub).*(hsub./(hsub+bwg))) + ((1./er_wg).*(bwg./(hsub+bwg))) );
end