function [wg] = WG_prop(a,b,freq,er,tand,mr,tanm)
% Function to calculate properties of a drectangular waveguide 
% Input: wg dimensions in [mm], freq in [GHz], filling permittivity 
% Return: wg sturcture with all nececery data in [mm], [GHz]
% 
% Written based on D. Pozar, "Microwave Engineering", ed. 4, chapter 3.3, Table 3.2
%
% 20201230 v1 J.Sorocki
%
%  |-----------|
% b|           |
%  |-----------|
%        a

if nargin < 6
    mr = 1;
    tanm = 0;
end 
if nargin < 4
    er = 1;
    tand = 0;
end

eps0 = 8.85418782*1e-12;%% 8.85418782 × 10-12 m-3 kg-1 s4 A2
mu0 = 1.25663706*1e-6; %1.25663706 × 10-6 m kg s-2 A-2
c0 = 1/sqrt(eps0*mu0);
eta0 = sqrt(mu0/eps0);
sigma_c=5.8e7;  % bulk conductivity of copper
Neper=8.686;    % dB to Neper

eta_sub = eta0*sqrt(mr/er);
f = freq.*1e9;
omega = 2.*pi.*f;
a = a.*1e-3;
b = b.*1e-3;

k = omega.*sqrt(eps0*mu0*mr*er);

% Calculation for TE10 mode
kc_10 = sqrt((1*pi/a)^2); %1/m
fc_10 = (c0/(2*pi*sqrt(er*mr)))*kc_10; %Hz
flow = 1.25.*fc_10;
fhigh = 1.9.*fc_10;
% disp(strcat('fc:',32,num2str(fc_10/1e9,3),32,'GHz'))
% disp(strcat('fl_TE10:',32,num2str(flow/1e9,3),32,'GHz'))
% disp(strcat('fh_TE10:',32,num2str(fhigh/1e9,3),32,'GHz'))
% disp(strcat('fm_TE10:',32,num2str(0.5*(fhigh+flow)/1e9,3),32,'GHz'))

beta_10 = sqrt((k.^2) - (kc_10^2));
lambdac_10 = 2*pi/kc_10;
lambdag_10 = 2.*pi./beta_10;
vp_10 = omega./beta_10;
alphad_10 = (k.*k.*tand)./(2.*beta_10);
Rs = sqrt((omega.*mu0)./(2.*sigma_c));
alphac_10 = (Rs.*(2.*b.*(pi.^2)+(a.^3).*(k.^2)))./((a.^3).*b.*beta_10.*k.*eta_sub); 


 % guide wave impedance throught different definitions
ZTE = (k.*eta_sub)./beta_10;
Zpi = ZTE.*((pi.*pi.*b)./(8.*a));
Zpv = ZTE.*((2.*b)./(a));
Zvi = ZTE.*((pi.*b)./(2.*a)); 

%attenuation in dB
alphad_10_dB = alphad_10.*Neper;    % dielectric related power loss
alphac_10_dB = alphac_10.*Neper;    % conductor related power loss
alphatot_10 = alphad_10 + alphac_10;

% TE20 mode
kc_20 = sqrt((2*pi/a)^2); %1/m
fc_20 = (c0/(2*pi*sqrt(er*mr)))*kc_20; %Hz

% via_d should be lower than lambdag/5
% via_p should be lower or equal to 2*d

wg.freq = freq;
wg.er = er;
wg.tand = tand;

wg.a = a*1e3;
wg.b = b*1e3;
wg.fc_10 = fc_10./1e9;
wg.fc_20 = fc_20./1e9;
wg.flow = flow./1e9;
wg.fhigh = fhigh./1e9;
wg.ZTE = ZTE;
wg.Z0 = Zpi;
wg.lambdag_10 = lambdag_10.*1e3;
wg.beta_10 = beta_10;
wg.alphad_10_dB = alphad_10_dB;
wg.alphac_10_dB = alphac_10_dB;

end